/*
 * say_es.c
 */

/*! \brief  say_number_full_es: Spanish syntax */
/* New files:
 Requires a few new audios:
   1F.gsm: feminine 'una'
   21.gsm thru 29.gsm, cien.gsm, mil.gsm, millon.gsm, millones.gsm, 100.gsm, 200.gsm, 300.gsm, 400.gsm, 500.gsm, 600.gsm, 700.gsm, 800.gsm, 900.gsm, y.gsm 
 */
static int say_number_full_es(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playa = 0;
	int mf = 0;                            /* +1 = male; -1 = female */
	char fn[256] = "";
	if (!num) 
		return say_digits_full(a, 0);

	if (options) {
		if (!strncasecmp(options, "f",1))
			mf = -1;
		else if (!strncasecmp(options, "m", 1))
			mf = 1;
	}

	while (!res && num) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playa) {
			snprintf(fn, sizeof(fn), "digits/and");
			playa = 0;
		} else if (num == 1) {
			if (mf < 0)
				snprintf(fn, sizeof(fn), "digits/%dF", num);
			else if (mf > 0)
				snprintf(fn, sizeof(fn), "digits/%dM", num);
			else 
				snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 31) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num/10)*10);
			num -= ((num/10)*10);
			if (num)
				playa++;
		} else if (num == 100) {
			snprintf(fn, sizeof(fn), "digits/100");
			num = 0;
		} else if (num < 200) {
			snprintf(fn, sizeof(fn), "digits/100-and");
			num -= 100;
		} else {
			if (num < ONE_THOUSAND) {
				snprintf(fn, sizeof(fn), "digits/%d", (num/100)*100);
				num -= ((num/100)*100);
			} else if (num < 2000) {
				num = num % ONE_THOUSAND;
				snprintf(fn, sizeof(fn), "digits/thousand");
			} else {
				if (num < ONE_MILLION) {
					res = say_number_full_es(a, num / ONE_THOUSAND, options);
					if (res)
						return res;
					num = num % ONE_THOUSAND;
					snprintf(fn, sizeof(fn), "digits/thousand");
				} else {
					if (num < 2147483640) {
						if ((num/ONE_MILLION) == 1) {
							res = say_number_full_es(a, num / ONE_MILLION, "M");
							if (res)
								return res;
							snprintf(fn, sizeof(fn), "digits/million");
						} else {
							res = say_number_full_es(a, num / ONE_MILLION, options);
							if (res)
								return res;
							snprintf(fn, sizeof(fn), "digits/millions");
						}
						num = num % ONE_MILLION;
					} else {
						ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
						res = -1;
					}
				}
			}
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Spanish syntax */
static int say_date_with_format_es(say_args_t *a, struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c, nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_es; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (format[offset]) {
			/* NOTE:  if you add more options here, please try to be consistent with strftime(3) */
			case '\'': /* Literal name of a sound file */
				res = play_literal(a->chan, a->ints, a->language, format, &offset);
				break;
			case 'A':
			case 'a':
				/* Sunday - Saturday */
				snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
				res = s_waitfile(a, nextmsg);
				break;
			case 'B':
			case 'b':
			case 'h':
				/* January - December */
				snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
				res = s_waitfile(a, nextmsg);
				break;
			case 'm':
				/* First - Twelfth */
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
				res = s_waitfile(a, nextmsg);
				break;
			case 'd':
			case 'e':
				/* First - Thirtyfirst */
				res = say_number_full_es(a, tm.tm_mday, NULL);
				break;
			case 'Y':
				/* Year */
				res = say_number_full_es(a, tm.tm_year + 1900, NULL);
				break;
			case 'I':
			case 'l':
				/* 12-Hour */
				snprintf(nextmsg,sizeof(nextmsg), "digits/%d", hr_to_12(tm.tm_hour));
				res = s_waitfile(a, nextmsg);
				break;
			case 'H':
			case 'k':
				/* 24-Hour */
				res = say_number_full_es(a, tm.tm_hour, NULL);
				break;
			case 'M':
				/* Minute */
				res = say_number_full_es(a, tm.tm_min, NULL);	
				break;
			case 'P':
			case 'p':
				/* AM/PM */
				if (tm.tm_hour > 12)
					res = s_waitfile(a, "digits/p-m");
				else if (tm.tm_hour  && tm.tm_hour < 12)
					res = s_waitfile(a, "digits/a-m");
				break;
			case 'Q': /* Shorthand for "Today", "Yesterday", or ABdY */
			case 'q': /* Shorthand for "" (today), "Yesterday", A (weekday), or ABdY */
					if (delta >= 0) { /* Today */
						if (c == 'Q')
							res = s_waitfile(a, "digits/today");
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = s_waitfile(a, "digits/yesterday");
					} else if (c == 'q' && delta >= - SIX_DAYS) {
						/* Within the last week */
						res = f(a, &tm, "A", delta);
					} else {
						res = f(a, &tm, "'digits/es-el' Ad 'digits/es-de' B 'digits/es-de' Y", delta);
					}
				break;
			case 'R':
				res = f(a, &tm, "H 'digits/y' M", delta);
				break;
			case 'S':
				/* Seconds */
				if (tm.tm_sec == 0) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = s_waitfile(a, nextmsg);
				} else if (tm.tm_sec < 10) {
					res = s_waitfile(a, "digits/oh");
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
						res = s_waitfile(a, nextmsg);
					}
				} else if ((tm.tm_sec < 21) || (tm.tm_sec % 10 == 0)) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = s_waitfile(a, nextmsg);
				} else {
					int ten, one;
					ten = (tm.tm_sec / 10) * 10;
					one = (tm.tm_sec % 10);
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", ten);
					res = s_waitfile(a, nextmsg);
					if (!res) {
						/* Fifty, not fifty-zero */
						if (one != 0) {
							snprintf(nextmsg,sizeof(nextmsg), "digits/%d", one);
							res = s_waitfile(a, nextmsg);
						}
					}
				}
				break;
			case 'T':
				res = f(a, &tm, "HMS", delta);
				break;
			case ' ':
			case '\t':
				/* Just ignore spaces and tabs */
				break;
			default:
				/* Unknown character */
				ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
		/* Jump out on DTMF */
		if (res) {
			break;
		}
	}
	return res;
}

static struct _lang_sw map_es = {
	.language = "es",
	.say_number_full = say_number_full_es,
	.say_date_with_format = say_date_with_format_es,
	.date_fmt = NULL, /* XXX */
	.time_fmt = NULL, /* XXX */
	.datetime_fmt = NULL, /* XXX */
};

