/*
 * say_he.c
 */

/*! \brief  say_number_full_he: Hebrew syntax */
/* 	Extra sounds needed:
 	1F: feminin 'one'
	ve: 'and'
	2hundred: 2 hundred
	2thousands: 2 thousand 
	thousands: plural of 'thousand'
	3sF 'Smichut forms (female)
	4sF
	5sF
	6sF
	7sF
	8sF
	9sF
	3s 'Smichut' forms (male)
	4s
	5s
	6s
	7s
	9s
	10s
	11s
	12s
	13s
	14s
	15s
	16s
	17s
	18s
	19s

TODO: 've' should sometimed be 'hu':
* before 'shtaym' (2, F)
* before 'shnaym' (2, M)
* before 'shlosha' (3, M)
* before 'shmone' (8, M)
* before 'shlosim' (30)
* before 'shmonim' (80)

What about:
'sheva' (7, F)?
'tesha' (9, F)?
*/
#define SAY_NUM_BUF_SIZE 256
static int say_number_full_he(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int state = 0; /* no need to save anything */
	int mf = 1;    /* +1 = Masculin; -1 = Feminin */
	char fn[SAY_NUM_BUF_SIZE] = "";
	ast_verbose(VERBOSE_PREFIX_3 "ast_say_digits_full: started. "
		"num: %d, options=\"%s\"\n",
		num, options
	);
	if (!num) 
		return say_digits_full(a, 0);
	
	if (options && !strncasecmp(options, "f",1))
		mf = -1;

	/* Do we have work to do? */
	while(!res && (num || (state>0) ))  {
		/* first type of work: play a second sound. In this loop
		 * we can only play one sound file at a time. Thus playing 
		 * a second one requires repeating the loop just for the 
		 * second file. The variable 'state' remembers where we were.
		 * state==0 is the normal mode and it means that we continue
		 * to check if the number num has yet anything left.
		 */
		ast_verbose(VERBOSE_PREFIX_3 "ast_say_digits_full: num: %d, "
			"state=%d, options=\"%s\", mf=%d\n",
			num, state, options, mf
		);
		if (state==1) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			state = 0;
		} else if (state==2) {
			snprintf(fn, sizeof(fn), "digits/ve");
			state = 0;
		} else if (state==3) {
			snprintf(fn, sizeof(fn), "digits/thousands");
			state=0;
		} else if (num <21) {
			if (mf < 0)
				snprintf(fn, sizeof(fn), "digits/%dF", num);
			else
				snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num/10)*10);
			num = num % 10;
			if (num>0) state=2;
		} else if (num < 200) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			num = num - 100;
		} else if (num < 300) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			num = num - 100;
		} else if (num < ONE_THOUSAND) {
			snprintf(fn, sizeof(fn), "digits/%d", (num/100));
			state=1;
			num = num % 100;
		} else if (num < 2000) {
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = num - ONE_THOUSAND;
		} else if (num < 3000) {
			snprintf(fn, sizeof(fn), "digits/2thousand");
			num = num - 2000;
                        if (num>0) state=2;
		} else if (num < 20000) {
			snprintf(fn, sizeof(fn), "digits/%ds",(num/ONE_THOUSAND));
			num = num % ONE_THOUSAND;
			state=3;
		} else if (num < ONE_MILLION) {
			res = say_number_full_he(a, num / ONE_THOUSAND, options);
			if (res)
				return res;
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = num % ONE_THOUSAND;
		} else	if (num < ONE_BILLION) {
			res = say_number_full_he(a, num / ONE_MILLION, options);
			if (res)
				return res;
			snprintf(fn, sizeof(fn), "digits/million");
			num = num % ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* TODO: this probably is not the correct format for doxygen remarks */

/** say_date_with_format_he Say formmated date in Hebrew
 *
 * \ref say_date_with_format_en for the details of the options 
 *
 * Changes from the English version: 
 *
 * * don't replicate in here the logic of say_number_full_he
 *
 * * year is always 4-digit (because it's simpler)
 *
 * * added c, x, and X. Mainly for my tests
 *
 * * The standard "long" format used in Hebrew is AdBY, rather than ABdY
 *
 * TODO: 
 * * A "ha" is missing in the standard date format, before the 'd'.
 * * The numbers of 3000--19000 are not handled well
 **/
#define IL_DATE_STR "AdBY"
#define IL_TIME_STR "IMp"
#define IL_DATE_STR_FULL IL_DATE_STR " 'digits/at' " IL_TIME_STR
static int say_date_with_format_he(say_args_t *a, struct tm *tm1,
    const char *format, time_t delta)
{
	/* TODO: This whole function is cut&paste from 
	 * say_date_with_format_en . Is that considered acceptable?
	 **/
	struct tm tm = *tm1;
	int res=0, offset;
	char c, nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_he; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (format[offset]) {
		case '\'': /* Literal name of a sound file */
			res = play_literal(a->chan, a->ints, a->language, format, &offset);
			break;
		case 'A':
		case 'a':
			/* Sunday - Saturday */
			snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
			res = s_waitfile(a, nextmsg);
			break;
		case 'B':
		case 'b':
		case 'h':
			/* January - December */
			snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
			res = s_waitfile(a,nextmsg);
			break;
		case 'd':
		case 'e': /* Day of the month */
			res = say_number_full_he(a, tm.tm_mday, "m");
			break;
		case 'Y': /* Year */
			res = say_number_full_he(a, tm.tm_year+1900, "f");
			break;
		case 'I':
		case 'l': /* 12-Hour */
			res = say_number_full_he(a, hr_to_12(tm.tm_hour), "f");
			break;
		case 'H':
		case 'k': /* 24-Hour */
			/* With 'H' there is an 'oh' after a single-digit hour */
			if (c == 'H' && tm.tm_hour <10 && tm.tm_hour>0 )
				res = s_waitfile(a, "digits/oh");
			res = say_number_full_he(a, tm.tm_hour, "f");
			break;
		case 'M': /* Minute */
			res = say_number_full_he(a, tm.tm_min, "f");
			break;
		case 'P':
		case 'p':
			/* AM/PM */
			res = s_waitfile(a, hr_to_am_pm(tm.tm_hour));
			break;
		case 'Q': /* Short for "Today", "Yesterday", or "date" */
		case 'q': /* Short for "" (today), "Yesterday", A (weekday), or "date" */
			if (delta >= 0) { /* Today */
				if (c == 'Q')
					res = s_waitfile(a, "digits/today");
			} else if (delta >= -ONE_DAY) { /* Yesterday */
				res = s_waitfile(a, "digits/yesterday");
			} else if (format[offset] != 'Q' && delta >= -SIX_DAYS) {
				/* Within the last week */
				res = f(a, &tm, "A", delta);
			} else {
				res = f(a, &tm, IL_DATE_STR, delta);
			}
			break;
		case 'R':
			res = f(a, &tm, "HM", delta);
			break;
		case 'S': /* Seconds */
			res = say_number_full_he(a, tm.tm_sec, "f");
			break;
		case 'T':
			res = f(a, &tm, "HMS", delta);
				break;
			/* c, x, and X seem useful for testing. Not sure
			 * if thiey're good for the general public */
		case 'c':
			res = f(a, &tm, IL_DATE_STR_FULL, delta);
			break;
		case 'x':
			res = f(a, &tm, IL_DATE_STR, delta);
			break;
		case 'X': /* Currently not locale-dependent...*/
			res = f(a, &tm, IL_TIME_STR, delta);
			break;
		case ' ':
		case '\t':
			/* Just ignore spaces and tabs */
			break;
		default:
			/* Unknown character */
			ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
	}
	return res;
}

static struct _lang_sw map_he = {
	.language = "he",
	.say_number_full = say_number_full_he,
	.say_date_with_format = say_date_with_format_he,
	.time_fmt = NULL, /* XXX */
	.date_fmt = NULL, /* XXX */
	.datetime_fmt = NULL, /* XXX */
};
