/*
 * say_it.c
 */

/*! \brief  say_number_full_it:  Italian */
static int say_number_full_it(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";

	if (!num)
		return say_digits_full(a, 0);

	/*
	Italian support

	Like english, numbers up to 20 are a single 'word', and others
	compound, but with exceptions.
	For example 21 is not twenty-one, but there is a single word in 'it'.
	Idem for 28 (ie when a the 2nd part of a compund number
	starts with a vowel)

	There are exceptions also for hundred, thousand and million.
	In english 100 = one hundred, 200 is two hundred.
	In italian 100 = cento , like to say hundred (without one),
	200 and more are like english.
	
	Same applies for thousand:
	1000 is one thousand in en, 2000 is two thousand.
	In it we have 1000 = mille , 2000 = 2 mila 

	For million(s) we use the plural, if more than one
	Also, one million is abbreviated in it, like on-million,
	or 'un milione', not 'uno milione'.
	So the right file is provided.
	*/

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < ONE_HUNDRED) {
			int units = num % 10;
			if (units == 1 || units == 8) { /* single word */
				snprintf(fn, sizeof(fn), "digits/%d", num);
				num = 0;
			} else {	/* two words */
				snprintf(fn, sizeof(fn), "digits/%d", num - units);
				num = units;
			}
		} else if (num < ONE_THOUSAND) {
			int hundreds = num / ONE_HUNDRED;
			if (hundreds == 1)
				snprintf(fn, sizeof(fn), "digits/hundred");
			else {
				snprintf(fn, sizeof(fn), "digits/%d", hundreds);
				playh++;
			}
			num %= ONE_HUNDRED;
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			int thousands = num/ONE_THOUSAND;
			if (thousands == 1)
				snprintf(fn, sizeof(fn), "digits/thousand");
			else {
				res = say_number_full_it(a, thousands, options);
				if (res)
					break;
				snprintf(fn, sizeof(fn), "digits/thousands");
			}
			num = num % ONE_THOUSAND;
		} else if (num < ONE_BILLION) { /* 1,000,000,000 */
			int millions = num / ONE_MILLION;
			if (millions == 1)
				snprintf(fn, sizeof(fn), "digits/million");
			else {
				res = say_number_full_it(a, millions, options);
				if (res)
					break;
				snprintf(fn, sizeof(fn), "digits/millions");
			}
			num = num % ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			break;
		}
		res = s_streamwait2(a, fn);
	}
	return res;
}

static int say_date_with_format_it(say_args_t *a, struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_it; /* shorthand for recursive call */
	char c;

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case '\'': /* Literal name of a sound file */
			res = play_literal(a->chan, a->ints, a->language, format, &offset);
			break;
		case 'A':
		case 'a':
			/* Sunday - Saturday */
			snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
			res = s_waitfile(a, nextmsg);
			break;
		case 'B':
		case 'b':
		case 'h':
			/* January - December */
			snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
			res = s_waitfile(a, nextmsg);
			break;
		case 'm':
			/* First - Twelfth */
			snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
			res = s_waitfile(a, nextmsg);
			break;
		case 'd':
		case 'e':
			/* First day of the month is spelled as ordinal */
			if (tm.tm_mday == 1) {
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday);
				res = s_waitfile(a, nextmsg);
			} else {
				if (!res)
					res = say_number_full_it(a, tm.tm_mday, NULL);
			}
			break;
		case 'Y':
			/* Year */
			res = say_number_full_it(a, tm.tm_year + 1900, NULL);
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			snprintf(nextmsg,sizeof(nextmsg), "digits/%d", hr_to_12(tm.tm_hour));
			res = s_waitfile(a, nextmsg);
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			if (tm.tm_hour == 0) {
				res = s_waitfile(a, "digits/ore-mezzanotte");
			} else if (tm.tm_hour == 1) {
				res = s_waitfile(a, "digits/ore-una");
			} else {
				res = say_number_full_it(a, tm.tm_hour, NULL);
			}
			break;
		case 'M':
			/* Minute */
			res = say_number_full_it(a, tm.tm_min, NULL);
			break;
		case 'P':
		case 'p':
			/* AM/PM */
			res = s_waitfile(a, hr_to_am_pm(tm.tm_hour));
			break;
		case 'Q': /* Short for "Today", "Yesterday", or ABdY */
		case 'q': /* Short for "" (today), "Yesterday", A (weekday), or ABdY */
			if (delta >= 0) { /* Today */
				if (c == 'Q')
					res = s_waitfile(a, "digits/today");
			} else if (delta >= -ONE_DAY) { /* Yesterday */
				res = s_waitfile(a, "digits/yesterday");
			} else if (c == 'q' && delta >= - SIX_DAYS) {
				/* Within the last week */
				res = f(a, &tm, "A", delta);
			} else {
				res = f(a, &tm, "AdB", delta);
			}
			break;
		case 'R':
			res = f(a, &tm, "HM", delta);
			break;
		case 'S':
			/* Seconds */
			res = say_number_full_it(a, tm.tm_sec, NULL);
			break;
		case 'T':
			res = f(a, &tm, "HMS", delta);
			break;
		case ' ':
		case '\t':
			/* Just ignore spaces and tabs */
			break;
		default:
			/* Unknown character */
			ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
	}
	return res;
}

static struct _lang_sw map_it = {
	.language = "it",
	.say_number_full = say_number_full_it,
	.say_date_with_format = say_date_with_format_it,
	.date_fmt = "AdBY",
	.time_fmt = "HM", /* XXX missing 'e' between H and M */
	.datetime_fmt = "AdBYHM", /* XXX missing 'ore' between date and time */
};

