/*
 * say_no.c
 */

/* New files:
 In addition to American English, the following sounds are required:  "and", "1N"
 */
/*! \brief  say_number_full_no: Norwegian syntax */
static int say_number_full_no(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	int playa = 0;
	int cn = 1;		/* +1 = commune; -1 = neuter */
	char fn[256] = "";
	
	if (!num) 
		return say_digits_full(a, 0);
	
	if (options && !strncasecmp(options, "n",1)) cn = -1;

	while(!res && (num || playh || playa )) {
		/* The grammar for Norwegian numbers is the same as for English except
		* for the following:
		* - 1 exists in both commune ("en", file "1") and neuter ("ett", file "1N")
		*   "and" before the last two digits, i.e. 2034 is "two thousand and
		*   thirty-four" and 1000012 is "one million and twelve".
		*/
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (playa) {
			snprintf(fn, sizeof(fn), "digits/and");
			playa = 0;
		} else if (num == 1 && cn == -1) {
			snprintf(fn, sizeof(fn), "digits/1N");
			num = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10);
		} else if (num < ONE_THOUSAND) {
			int hundreds = num / 100;
			if (hundreds == 1)
				snprintf(fn, sizeof(fn), "digits/1N");
			else
				snprintf(fn, sizeof(fn), "digits/%d", (num / 100));

			playh++;
			num -= 100 * hundreds;
			if (num)
				playa++;
		} else 	if (num < ONE_MILLION) {
			res = say_number_full_no(a, num / ONE_THOUSAND, "n");
			if (res)
				return res;
			snprintf(fn, sizeof(fn), "digits/thousand");
			num = num % ONE_THOUSAND;
			if (num && num < 100)
				playa++;
		} else 	if (num < ONE_BILLION) {
			int millions = num / ONE_MILLION;
			res = say_number_full_no(a, millions, "c");
			if (res)
				return res;
			snprintf(fn, sizeof(fn), "digits/million");
			num = num % ONE_MILLION;
			if (num && num < 100)
				playa++;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

static struct _lang_sw map_no = {
	.language = "no",
	0,
	.say_number_full = say_number_full_no,
};

