/*
 * say_pt.c
 */

/*
 ****************** PORTUGUESE **************************
 */

/* say_number_full_pt: Portuguese syntax */
/* 	Extra sounds needed: */
/* 	For feminin all sound files end with F */
/*	100E for 100+ something */
/*	1000000S for plural */
/*	pt-e for 'and' */
static int say_number_full_pt(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	int mf = 1;                            /* +1 = male; -1 = female */
	char fn[256] = "";

	if (!num) 
		return say_digits_full(a, 0);

	if (options && !strncasecmp(options, "f",1))
		mf = -1;

	while(!res && num ) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (num < 20) {
			if ((num == 1 || num == 2) && (mf < 0))
				snprintf(fn, sizeof(fn), "digits/%dF", num);
			else
				snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num / 10) * 10);
			if (num % 10)
				playh = 1;
			num = num % 10;
		} else if (num < ONE_THOUSAND) {
			if (num == 100)
				snprintf(fn, sizeof(fn), "digits/100");
			else if (num < 200)
				snprintf(fn, sizeof(fn), "digits/100E");
			else {
				if (mf < 0 && num > 199)
					snprintf(fn, sizeof(fn), "digits/%dF", (num / 100) * 100);
				else
					snprintf(fn, sizeof(fn), "digits/%d", (num / 100) * 100);
				if (num % 100)
					playh = 1;
			}
			num = num % 100;
		} else if (num < ONE_MILLION) {
			if (num > 1999) {
				res = say_number_full_pt(a, (num / ONE_THOUSAND) * mf, options);
				if (res)
					return res;
			}
			snprintf(fn, sizeof(fn), "digits/1000");
			if ((num % ONE_THOUSAND) && ((num % ONE_THOUSAND) < 100  || !(num % 100)))
				playh = 1;
			num = num % ONE_THOUSAND;
		} else if (num < ONE_BILLION) {
			res = say_number_full_pt(a, (num / ONE_MILLION), options);
			if (res)
				return res;
			if (num < 2000000)
				snprintf(fn, sizeof(fn), "digits/1000000");
			else
				snprintf(fn, sizeof(fn), "digits/1000000S");
 
			if ((num % ONE_MILLION) &&
				/* no thousands */
				((!((num / ONE_THOUSAND) % ONE_THOUSAND) && ((num % ONE_THOUSAND) < 100 || !(num % 100))) ||
				/* no hundreds and below */
				(!(num % ONE_THOUSAND) && (((num / ONE_THOUSAND) % ONE_THOUSAND) < 100 || !((num / ONE_THOUSAND) % 100))) ) )
				playh = 1;
			num = num % ONE_MILLION;
		}
		if (!res)
			res = s_streamwait3(a, fn);
		if (!res && playh) {
			res = wait_file(a->chan, a->ints, "digits/pt-e", a->language);
			ast_stopstream(a->chan);
			playh = 0;
		}
	}
	return res;
}

/* Portuguese syntax */
static int say_date_with_format_pt(say_args_t *a, struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c, nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_pt; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (format[offset]) {
		case '\'': /* Literal name of a sound file */
			res = play_literal(a->chan, a->ints, a->language, format, &offset);
			break;
		case 'A':
		case 'a':
			/* Sunday - Saturday */
			snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
			res = s_waitfile(a, nextmsg);
			break;
		case 'B':
		case 'b':
		case 'h':
			/* January - December */
			snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
			res = s_waitfile(a, nextmsg);
			break;
		case 'm':
			/* First - Twelfth */
			snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
			res = s_waitfile(a, nextmsg);
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */
			res = say_number_full_pt(a, tm.tm_mday, NULL);
			break;
		case 'Y':
			/* Year */
			res = say_number_full_pt(a, tm.tm_year + 1900, NULL);
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			if (tm.tm_hour == 0) {
				if (format[offset] == 'I')
					res = s_waitfile(a, "digits/pt-ah");
				if (!res)
					res = s_waitfile(a, "digits/pt-meianoite");
			} else if (tm.tm_hour == 12) {
				if (format[offset] == 'I')
					res = s_waitfile(a, "digits/pt-ao");
				if (!res)
					res = s_waitfile(a, "digits/pt-meiodia");
			} else {
				if (format[offset] == 'I') {
					res = s_waitfile(a, "digits/pt-ah");
					if ((tm.tm_hour % 12) != 1)
						if (!res)
							res = s_waitfile(a, "digits/pt-sss");
				}
				if (!res)
					res = say_number_full_pt(a, tm.tm_hour % 12, "f");
			}
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			/* XXX what is this ? */
			res = ast_say_number(a->chan, -tm.tm_hour,a->ints, a->language, NULL);
			if (!res) {
				if (tm.tm_hour != 0) {
					int remainder = tm.tm_hour;
					if (tm.tm_hour > 20) {
						res = s_waitfile(a, "digits/20");
						remainder -= 20;
					}
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", remainder);
						res = s_waitfile(a, nextmsg);
					}
				}
			}
			break;
		case 'M':
			/* Minute */
			if (tm.tm_min == 0) {
				res = s_waitfile(a, "digits/pt-hora");
				if (tm.tm_hour != 1)
					if (!res)
						res = s_waitfile(a, "digits/pt-sss");
			} else {
				res = s_waitfile(a, "digits/pt-e");
				if (!res)
					res = say_number_full_pt(a, tm.tm_min, NULL);	
			}
			break;
		case 'P':
		case 'p':
			/* AM/PM */
			if (tm.tm_hour > 12)
				res = s_waitfile(a, "digits/p-m");
			else if (tm.tm_hour  && tm.tm_hour < 12)
				res = s_waitfile(a, "digits/a-m");
			break;
		case 'Q': /* Short for "Today", "Yesterday", or ABdY */
		case 'q': /* Short for "" (today), "Yesterday", A (weekday), or ABdY */
			if (delta >= 0) { /* Today */
				res = s_waitfile(a, "digits/today");
			} else if (delta >= -ONE_DAY) { /* Yesterday */
				res = s_waitfile(a, "digits/yesterday");
			} else if (c == 'q' && delta >= - SIX_DAYS) {
				/* Within the last week */
				res = f(a, &tm, "A", delta);
			} else {
				res = f(a, &tm, "Ad 'digits/pt-de' B 'digits/pt-de' Y", delta);
			}
			break;
		case 'R':
			res = f(a, &tm, "H 'digits/pt-e' M", delta);
			break;
		case 'S': /* Seconds */
			if (tm.tm_sec > 0 && tm.tm_sec < 10)
				res = s_waitfile(a, "digits/oh");
			if (!res)
				res = say_number_full_pt(a, tm.tm_sec, NULL);
			break;
		case 'T':
			res = f(a, &tm, "HMS", delta);
			break;
		case ' ':
		case '	':
			/* Just ignore spaces and tabs */
			break;
		default:
			/* Unknown character */
			ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
	}
	return res;
}

/* Portuguese syntax */

static int say_datetime_from_now_pt(say_args_t *a, time_t t)
{
	int res=0;
	time_t nowt;
	int daydiff;
	struct tm tm;
	struct tm now;
	char fn[256];

	time(&nowt);

	localtime_r(&t,&tm);
	localtime_r(&nowt,&now);
	daydiff = now.tm_yday - tm.tm_yday;
	if ((daydiff < 0) || (daydiff > 6)) {
		/* Day of month and month */
		if (!res)
			res = say_number_full_pt(a, tm.tm_mday, NULL);
		if (!res)
			res = s_waitfile(a, "digits/pt-de");
		snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
		if (!res)
			res = s_waitfile(a, fn);
	
	} else if (daydiff) {
		/* Just what day of the week */
		snprintf(fn, sizeof(fn), "digits/day-%d", tm.tm_wday);
		if (!res)
			res = s_waitfile(a, fn);
	}	/* Otherwise, it was today */
	snprintf(fn, sizeof(fn), "digits/pt-ah");
	if (!res)
		res = s_waitfile(a, fn);
	if (tm.tm_hour != 1)
		if (!res)
			res = s_waitfile(a, "digits/pt-sss");
	if (!res)
		res = ast_say_time(a->chan, t, a->ints, a->language);
	return res;
}


static struct _lang_sw map_pt = {
	.language = "pt",
	.say_number_full = say_number_full_pt,
	.say_date_with_format = say_date_with_format_pt,
	.say_datetime_from_now = say_datetime_from_now_pt,
	.date_fmt = "Ad 'digits/pt-de' B 'digits/pt-de' Y",
	.time_fmt = "H 'digits/pt-e' M",
	.datetime_fmt ="ABdIMPY",
};

