/*
 * say_ru.c
 */

/*! \brief  determine last digits for thousands/millions (ru) */
static int get_lastdigits_ru(int num) {
	if (num < 20) {
		return num;
	} else if (num < 100) {
		return get_lastdigits_ru(num % 10);
	} else if (num < ONE_THOUSAND) {
		return get_lastdigits_ru(num % 100);
	}
	return 0;	/* number too big */
}


/*! \brief  say_number_full_ru: Russian syntax */
/*! \brief  additional files:
	n00.gsm			(one hundred, two hundred, ...)
	thousand.gsm
	million.gsm
	thousands-i.gsm		(tisyachi)
	million-a.gsm		(milliona)
	thousands.gsm
	millions.gsm
	1f.gsm			(odna)
	2f.gsm			(dve)
    
	where 'n' from 1 to 9
*/
static int say_number_full_ru(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int lastdigits = 0;
	char fn[256] = "";
	if (!num) 
		return say_digits_full(a, 0);

	while(!res && (num)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else	if (num < 20) {
			if(options && strlen(options) == 1 && num < 3) {
			    snprintf(fn, sizeof(fn), "digits/%d%s", num, options);
			} else {
    			    snprintf(fn, sizeof(fn), "digits/%d", num);
			}
			num = 0;
		} else	if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", num - (num % 10));
			num %= 10;
		} else 	if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", num - (num % 100));
			num %= 100;
		} else 	if (num < ONE_MILLION) { /* 1,000,000 */
			lastdigits = get_lastdigits_ru(num / ONE_THOUSAND);
			/* say thousands */
			if (lastdigits < 3) {
				res = say_number_full_ru(a, num / ONE_THOUSAND, "f");
			} else {
				res = say_number_full_ru(a, num / ONE_THOUSAND, NULL);
			}
			if (res)
				return res;
			if (lastdigits == 1) {
				snprintf(fn, sizeof(fn), "digits/thousand");
			} else if (lastdigits > 1 && lastdigits < 5) {
				snprintf(fn, sizeof(fn), "digits/thousands-i");
			} else {
				snprintf(fn, sizeof(fn), "digits/thousands");
			}
			num %= ONE_THOUSAND;
		} else 	if (num < ONE_BILLION) {	/* 1,000,000,000 */
			lastdigits = get_lastdigits_ru(num / ONE_MILLION);
			/* say millions */
			res = say_number_full_ru(a, num / ONE_MILLION, NULL);
			if (res)
				return res;
			if (lastdigits == 1) {
				snprintf(fn, sizeof(fn), "digits/million");
			} else if (lastdigits > 1 && lastdigits < 5) {
				snprintf(fn, sizeof(fn), "digits/million-a");
			} else {
				snprintf(fn, sizeof(fn), "digits/millions");
			}
			num %= ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
				res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

static struct _lang_sw map_ru = {
	.language = "ru",
	0,
	.say_number_full = say_number_full_ru,
};

