/*
 * say_se.c
 */

/*! \brief  say_number_full_se: Swedish syntax */
static int say_number_full_se(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";
	int cn = 1;		/* +1 = commune; -1 = neuter */
	if (!num) 
		return say_digits_full(a, 0);
	if (options && !strncasecmp(options, "n",1)) cn = -1;

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10);
		} else if (num == 1 && cn == -1) {	/* En eller ett? */
		 	snprintf(fn, sizeof(fn), "digits/1N");
			num = 0;
		} else if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", num/100);
			playh++;
			num %= 100;
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			res = say_number_full_se(a, num / ONE_THOUSAND, options);
			if (res)
				break;
			num = num % ONE_THOUSAND;
			snprintf(fn, sizeof(fn), "digits/thousand");
		} else if (num < ONE_BILLION) {	/* 1,000,000,000 */
			res = say_number_full_se(a, num / ONE_MILLION, options);
			if (res)
				return res;
			num = num % ONE_MILLION;
			snprintf(fn, sizeof(fn), "digits/million");
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

static struct _lang_sw map_se = {
	.language = "se",
	.say_number_full = say_number_full_se,
};

