/*
 * say_tw.c
 */

/*! \brief  say_number_full_tw: Taiwanese syntax */
static int say_number_full_tw(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";
	if (!num)
		return say_digits_full(a, 0);

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else	if (num < 10) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else	if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10);
		} else if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", (num/100));
			playh++;
			num -= ((num / 100) * 100);
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			res = say_number_full_tw(a, num / ONE_THOUSAND, options);
			if (res)
				return res;
			num = num % ONE_THOUSAND;
			snprintf(fn, sizeof(fn), "digits/thousand");
		} else if (num < ONE_BILLION) {	/* 1,000,000,000 */
			res = say_number_full_tw(a, num / ONE_MILLION, options);
			if (res)
				return res;
			num = num % ONE_MILLION;
			snprintf(fn, sizeof(fn), "digits/million");
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Taiwanese syntax */
static int say_date_with_format_tw(say_args_t *a, struct tm *tm1,
	const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char c, nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_tw; /* shorthand for recursive call */

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case '\'': /* Literal name of a sound file */
			res = play_literal(a->chan, a->ints,a->language, format, &offset);
			break;
		case 'A':
		case 'a':
			/* Sunday - Saturday */
			snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
			res = s_waitfile(a, nextmsg);
			break;
		case 'B':
		case 'b':
		case 'h':
			/* January - December */
			snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
			res = s_waitfile(a, nextmsg);
			break;
		case 'm':
			/* First - Twelfth */
			snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
			res = s_waitfile(a, nextmsg);
			break;
		case 'd':
		case 'e':
			/* First - Thirtyfirst */
			if (!(tm.tm_mday % 10) || (tm.tm_mday < 10)) {
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday);
				res = s_waitfile(a, nextmsg);
			} else {
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%dh", tm.tm_mday - (tm.tm_mday % 10));
				res = s_waitfile(a, nextmsg);
				if(!res) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday % 10);
					res = s_waitfile(a, nextmsg);
				}
			}
			break;
		case 'Y':
			/* Year */
			if (tm.tm_year > 99) {
				res = s_waitfile(a,  "digits/2");
				if (!res) {
					res = s_waitfile(a,  "digits/thousand");
				}
				if (tm.tm_year > 100) {
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", (tm.tm_year - 100) / 10);
						res = s_waitfile(a, nextmsg);
						if (!res) {
							snprintf(nextmsg,sizeof(nextmsg), "digits/%d", (tm.tm_year - 100) % 10);
							res = s_waitfile(a, nextmsg);
							}
						}
					}
					if (!res) {
						res = s_waitfile(a,  "digits/year");
					}
				} else {
					if (tm.tm_year < 1) {
						/* I'm not going to handle 1900 and prior */
						/* We'll just be silent on the year, instead of bombing out. */
					} else {
						res = s_waitfile(a,  "digits/1");
						if (!res) {
							res = s_waitfile(a,  "digits/9");
						}
						if (!res) {
							if (tm.tm_year <= 9) {
								/* 1901 - 1909 */
								res = s_waitfile(a,  "digits/0");
								if (!res) {
									snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year);
									res = s_waitfile(a, nextmsg);
								}
							} else {
								/* 1910 - 1999 */
								snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year / 10);
								res = s_waitfile(a, nextmsg);
								if (!res) {
									snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year % 10);
									res = s_waitfile(a, nextmsg);
								}
							}
						}
					}
					if (!res) {
						res = s_waitfile(a,  "digits/year");
					}
				}
				break;
			case 'I':
			case 'l':
				/* 12-Hour */
				snprintf(nextmsg,sizeof(nextmsg), "digits/%d", hr_to_12(tm.tm_hour));
				res = s_waitfile(a, nextmsg);
				if (!res) {
					res = s_waitfile(a,  "digits/oclock");
				}
				break;
			case 'H':
			case 'k':
				/* 24-Hour */
				if (!(tm.tm_hour % 10) || tm.tm_hour < 10) {
					if (tm.tm_hour < 10) {
						res = s_waitfile(a, "digits/0");
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour);
					res = s_waitfile(a, nextmsg);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour - (tm.tm_hour % 10));
					res = s_waitfile(a, nextmsg);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour % 10);
						res = s_waitfile(a, nextmsg);
					}
				}
				if (!res) {
					res = s_waitfile(a,  "digits/oclock");
				}
				break;
			case 'M':
				/* Minute */
				if (!(tm.tm_min % 10) || tm.tm_min < 10) {
					if (tm.tm_min < 10) {
						res = s_waitfile(a, "digits/0");
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min);
					res = s_waitfile(a, nextmsg);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min - (tm.tm_min % 10));
					res = s_waitfile(a, nextmsg);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min % 10);
						res = s_waitfile(a, nextmsg);
					}
				}
				if (!res) {
					res = s_waitfile(a,  "digits/minute");
				}
				break;
			case 'P':
			case 'p':
				/* AM/PM */
				res = s_waitfile(a, hr_to_am_pm(tm.tm_hour));
				break;
			case 'Q':
				/* Shorthand for "Today", "Yesterday", or ABdY */
					if (delta >= 0) { /* Today */
						res = s_waitfile(a,  "digits/today");
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = s_waitfile(a,  "digits/yesterday");
					} else {
						res = f(a, &tm, "YBdA", delta);
					}
				break;
			case 'q':
				/* Shorthand for "" (today), "Yesterday", A (weekday), or ABdY */
					if (delta >= 0) { /* Today */
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = s_waitfile(a,  "digits/yesterday");
					} else if (delta >= - SIX_DAYS) {
						/* Within the last week */
						res = f(a, &tm, "A", delta);
					} else {
						res = f(a, &tm, "YBdA", delta);
					}
				break;
			case 'R':
				res = f(a, &tm, "HM", delta);
				break;
			case 'S':
				/* Seconds */
				if (!(tm.tm_sec % 10) || tm.tm_sec < 10) {
					if (tm.tm_sec < 10) {
						res = s_waitfile(a, "digits/0");
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = s_waitfile(a, nextmsg);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec - (tm.tm_sec % 10));
					res = s_waitfile(a, nextmsg);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec % 10);
						res = s_waitfile(a, nextmsg);
					}
				}
				if (!res) {
					res = s_waitfile(a,  "digits/second");
				}
				break;
			case 'T':
				res = f(a, &tm, "HMS", delta);
				break;
			case ' ':
			case '	':
				/* Just ignore spaces and tabs */
			break;
			default:
				/* Unknown character */
				ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
		/* Jump out on DTMF */
		if (res) {
			break;
		}
	}
	return res;
}

static struct _lang_sw map_tw = {
	.language = "tw",
	.say_number_full = say_number_full_tw,
	.say_date_with_format = say_date_with_format_tw,
	.time_fmt = "PIM",
	.datetime_fmt = "YBdAPIM",
};

